/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor.greenseeker;

import org.openide.util.NbPreferences;

/**
 * Greenseeker preferences, wraps the NbPreferences.forModule preference loader
 *
 * @author Gerben Feenstra
 */
public enum GreenseekerPreference {

    SENSOR_DATA_FILE_FORMAT("sensorDataFileFormat", "CYYYYMMDDCCCCCC.dbf"),
    GREENSEEKER_SENSOR_HEIGTH("greenseekerSensorHeight", "0"),
    NUMBER_OF_GREENSEEKER_SENSORS("numberOfGreenseekerSensors", "0"),
    GREENSEEKER_SENSOR_SPREAD("greenseekerSensorSpread", ""),
    SENSOR_TYPE("sensorType", ""),
    SPECTRAL_CHANNELS("spectralRange", ""),
    DELIMITER("recordDelimiter", " ");
        
    private String propertyName;
    private String defaultProperty;

    private GreenseekerPreference(String propertyName, String defaultProperty) {
        this.propertyName = propertyName;
        this.defaultProperty = defaultProperty;
    }

    public String getDefaultProperty() {
        return defaultProperty;
    }

    public String getPropertyName() {
        return propertyName;
    }

    public String getValue() {
        return NbPreferences.forModule(this.getClass()).get(this.getPropertyName(), this.getDefaultProperty());
    }

    public void saveValue(String value) {
        NbPreferences.forModule(this.getClass()).put(this.getPropertyName(), value);
    }
}
