/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor.greenseeker;

import com.linuxense.javadbf.DBFException;
import com.linuxense.javadbf.DBFField;
import com.linuxense.javadbf.DBFReader;
import java.awt.BorderLayout;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import javax.swing.*;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.openide.awt.UndoRedo;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;

/**
 * Creates a structure view of Greenseeker sensor files. Displays sensor values
 * in a
 *
 * @author Arjan Loermans
 */
@MultiViewElement.Registration(displayName = "#LBL_GreenseekerStructureView",
iconBase = "nl/cloudfarming/client/sensor/greenseeker/greenseeker-file-icon.png",
mimeType = GreenseekerDataObject.MIME_TYPE,
persistenceType = TopComponent.PERSISTENCE_NEVER,
preferredID = "GreenseekerStructure",
position = 300)
@NbBundle.Messages({
    "LBL_GreenseekerStructureView=Structure"
})
public class GreenseekerStructureView extends JPanel implements MultiViewElement {

    private JToolBar toolbar = new JToolBar();
    private GreenseekerDataObject obj;
    private MultiViewElementCallback callback;

    public GreenseekerStructureView(Lookup lkp) {
        obj = lkp.lookup(GreenseekerDataObject.class);
        assert obj != null;

        setLayout(new BorderLayout());
        JTable sensorTable = new JTable(getContents(), getColumnNames());

        JScrollPane scrollPane = new JScrollPane(sensorTable);
        sensorTable.setFillsViewportHeight(true);
        add(scrollPane);
    }

    public void updateName() {
        if (callback != null) {
            String name = getLookup().lookup(GreenseekerDataObject.class).getName();
            TopComponent tc = callback.getTopComponent();
            tc.setHtmlDisplayName(name);
            tc.setDisplayName(name);
            tc.setName(getName());
            tc.setToolTipText(getToolTipText());
        }
    }

    @Override
    public JComponent getVisualRepresentation() {
        return this;
    }

    @Override
    public JComponent getToolbarRepresentation() {
        return toolbar;
    }

    @Override
    public Action[] getActions() {
        return new Action[0];
    }

    @Override
    public Lookup getLookup() {
        return obj.getLookup();
    }

    @Override
    public void componentOpened() {
    }

    @Override
    public void componentClosed() {
    }

    @Override
    public void componentShowing() {
        if (callback != null) {
            updateName();
        }
    }

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentActivated() {
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public UndoRedo getUndoRedo() {
        return UndoRedo.NONE;
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback mvec) {
        this.callback = mvec;
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }

    private String[] getColumnNames() {
        FileObject fileObject = obj.getPrimaryFile();
        InputStream inputStream;
        String retVal[] = null;
        try {
            inputStream = new FileInputStream(FileUtil.toFile(fileObject));
            DBFReader reader = new DBFReader(inputStream);
            int numberOfFields = reader.getFieldCount();
            retVal = new String[numberOfFields];
            for (int i = 0; i < numberOfFields; i++) {
                DBFField field = reader.getField(i);
                retVal[i] = field.getName();
            }
        } catch (DBFException | FileNotFoundException ex) {
            Exceptions.printStackTrace(ex);
        }
        return retVal;
    }

    private String[][] getContents() {
        FileObject fileObject = obj.getPrimaryFile();
        InputStream inputStream;
        String[][] retVal = null;
        try {
            inputStream = new FileInputStream(FileUtil.toFile(fileObject));
            DBFReader reader = new DBFReader(inputStream);
            int nrOfFields = reader.getFieldCount();
            int nrOfRecords = reader.getRecordCount(); // We skip the first field, as this is the ObjectID
            retVal = new String[nrOfRecords][nrOfFields];
            for (int row = 0; row < nrOfRecords; row++) {
                Object[] rowObject = reader.nextRecord();
                for (int field = 0; field < nrOfFields; field++) {
                    retVal[row][field] = rowObject[field].toString();
                }
            }
        } catch (DBFException | FileNotFoundException ex) {
            Exceptions.printStackTrace(ex);
        }
        return retVal;
    }
}